import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

public class Enemy extends Enemies
{
    private int moveDirection = 1;
    private int stepCount = 0;
    private int timeInRange = 0;

    private static GreenfootImage[] leftImages;
    private static GreenfootImage[] rightImages;

    private static final int IMAGECOUNT = 10;
    private static final int ANIMATERATE = 5;
    private static final int AGGRORANGE = 250;
    private static final int REFIRERATE = 30;
    private static final int MOVESPEED = 3;
    
    private int healthTotal = 100;

    public Enemy()
    {
        createImages();
    }

    public void createImages()
    {
        if (leftImages == null || rightImages == null)
        {
            rightImages = new GreenfootImage[IMAGECOUNT];
            leftImages = new GreenfootImage[IMAGECOUNT];
            for (int i = 0; i < rightImages.length; i++)
            {
                String imageName = "Enemy" + i + ".png";
                rightImages[i] = new GreenfootImage(imageName);
                leftImages[i] = new GreenfootImage(imageName);
                leftImages[i].mirrorHorizontally();
            }
        }
    }

    /**
     * Act - do whatever the Enemy2 wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        scrollingMethods(); //This should be in the act method of all scrolling actors.
        checkHorizontalCollision();
        checkForPlayer();
        checkForFire();
    }    
    
    public void checkForFire()
    {
        ScrollingWorld theWorld = (ScrollingWorld) getWorld();
        Actor fire = getOneIntersectingObject(Fire.class);
        if (fire != null)
        {
            healthTotal = healthTotal - 20;
            theWorld.removeObject(fire);
        }
        if (healthTotal < 0)
        {
            theWorld.addObject(new Explosion(),getX(), getY());
            theWorld.removeObject(this);
        }
    }

    public void checkForPlayer()
    {
        timeInRange++;
        ScrollingWorld theWorld = (ScrollingWorld) getWorld();
        if (getObjectsInRange(AGGRORANGE, Player.class).size() > 0)
        {
            if (theWorld.getPlayer().getX() > getX())
            {
                setImage(rightImages[0]);
            }
            else
            {
                setImage(leftImages[0]);
            }
            if (timeInRange > REFIRERATE)
            {
                fire();
                timeInRange = 0;
            }
        }
        else
        {
            patrol();
            animate();
        }
    }

    public void patrol()
    {
        move(MOVESPEED * moveDirection);
    }

    public void animate()
    {
        stepCount++;
        int imageIndex = stepCount / ANIMATERATE;
        if (stepCount <= ANIMATERATE * IMAGECOUNT - 1)
        {
            if (moveDirection > 0)
            {
                setImage(rightImages[imageIndex]);
            }
            if (moveDirection < 0)
            {
                setImage(leftImages[imageIndex]);
            }
        }
        else
        {
            stepCount = 5;
        }
    }

    private void fire()
    {
        ScrollingWorld theWorld = (ScrollingWorld) getWorld();
        if (getImage().equals(rightImages[0]))
        {
            theWorld.addObject(new Fire(1, "RED"), getX() + 40, getY());
        }
        if (getImage().equals(leftImages[0]))
        {
            theWorld.addObject(new Fire(-1, "RED"), getX() - 40, getY());
        }
    }

    public void checkHorizontalCollision()
    {
        checkRightCollision();
        checkLeftCollision();
    }

    public void checkRightCollision()
    {
        int checkXDistance = 0;
        checkXDistance = (2 + (getImage().getWidth() / 2));
        Actor inWay = getOneObjectAtOffset(checkXDistance, 0, Ground.class);
        Actor inWay2 = getOneObjectAtOffset(checkXDistance, -30, Ground.class);
        Actor inWay3 = getOneObjectAtOffset(checkXDistance, 30, Ground.class);
        Actor inWay4 = getOneObjectAtOffset(checkXDistance / 2, (getImage().getHeight() / 2 + 10), Ground.class);
        if (inWay == null && inWay2 == null && inWay3 == null)
        {
            if (inWay4 == null)
            {
                moveDirection = -1;
            }
        }
        else
        {
            moveDirection = -1;
        }
    }

    public void checkLeftCollision()
    {
        int checkDistance = 0;
        checkDistance = (getImage().getWidth() / 2);
        Actor inWay = getOneObjectAtOffset(-checkDistance, 0, Ground.class);
        Actor inWay2 = getOneObjectAtOffset(-checkDistance, -30, Ground.class);
        Actor inWay3 = getOneObjectAtOffset(-checkDistance, 30, Ground.class);
        Actor inWay4 = getOneObjectAtOffset(-checkDistance / 2, (getImage().getHeight() / 2 + 10), Ground.class);
        if (inWay == null && inWay2 == null && inWay3 == null)
        {
            if (inWay4 == null)
            {
                moveDirection = 1;
            }
        }
        else
        {
            moveDirection = 1;
        }
    }
}
