import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.io.*;
import java.util.Scanner;
import java.util.List;

/**
 * Write a description of class ScrollingWorld here.
 * 
 */
public class ScrollingWorld extends World
{
    private int leftX; // leftmost x coordinate for a tile
    private int topY;  // highest y coordinate for a tile
    public Player thePlayer;
    public Objects worldObjects;
    public LifeCounter theLifeCounter;

    private static final int BACKGROUNDWIDTH = 1214; 
    private static final int MAP_HEIGHT = 15; //Height of map in number of tiles.
    private static final int TILE_WIDTH = 40;  //Width of tiles in pixels.
    private static final int TILE_HEIGHT = 40;  //Height of tiles in pixels.

    public boolean shouldScroll = true;
    public int scrollDirection = 0;

    public boolean checkpointReached = false;
    public static final int CHECKPOINTSHIFT = 6740;
    public boolean inControl = false;
    public boolean titleScreen = false;
    public int introPanel = 0;
    public int introTimer = 0;

    public GreenfootSound TitleSong = new GreenfootSound("Halo.mid");
    public GreenfootSound StageOneSong = new GreenfootSound("run.mid");

    /**
     * set of variables that change during gameplay.
     */
    public int dataCores = 0;
    public int totalLives = 3;
    public int stageNumber = 1;

    /**
     * Whether dataCores should be added to world upon creation.
     * Changes to false once it has been collected.
     */
    public boolean dataCore1 = true;
    public boolean dataCore2 = true;
    public boolean dataCore3 = true;
    public boolean dataCore4 = true;
    public boolean dataCore5 = true;       
    
    

    public ScrollingWorld()
    {    
        super(800, 600, 1, false); //The extra 'false' parameter means that the world is unbounded.
        setPaintOrder(Credits.class, MainInstructions.class, Cover.class, Statics.class, Front.class, Player.class, 
            Enemies.class, Ground.class, Clouds1.class, Pelican.class, Back.class, Background.class, FarBackground.class);
        
        title();
    }

    public void title()
    {
        titleScreen = true;
        TitleSong.stop();
        StageOneSong.stop();
        setBackground("titlebackground.png");
        clearObjects();
        addObject(new Cover(255, -1), 400, 300);
        //addObject(new MainInstructions(), 400, 300);
    }

    public void introSequence()
    {
        Greenfoot.delay(50);
        clearObjects();
        setBackground("ringworld.jpg");
        introOne();
    }

    public void introOne()
    {
        introPanel = 1;
        introTimer = 0;
        clearObjects();
        setBackground("ringworld.jpg");
        addObject(new Cover1(2), 400, 300);
        addObject(new Pelican(388, 136, 50), 175, 250);
        addObject(new Clouds(), 200, 600);
        addObject(new Clouds1(), 464, 535);
        addObject(new Credits(0), 400, 300);
    }

    public void animateIntroOne()
    {
        if (introTimer == 200)
        {
            addObject(new Clouds1(), 650, 550);
        }
    }

    public void introTwo()
    {
        introPanel = 2;
        clearObjects();
        setPaintOrder(Credits.class, Cover.class, Statics.class, Front.class, Player.class, Enemies.class, Ground.class, 
            Back.class, Warthog.class, Pelican.class, Background.class, FarBackground.class);
        setBackground("ringworld.jpg");
        addObject(new Pelican(1008, 360, 10), 392, 163);
        addObject(new Warthog(10), 207, 149);
        addObject(new Cover1(), 400, 300);
        introTwoObjects();
        for (int i = 0; i < 15; i++)
        {
            addObject(new Tile(98), 120 + (i * 40), 290);
            addObject(new Tile2(), 120 + (i * 40), 250);
        }
        addObject(new Credits(3), 400, 300);
        addObject(new Credits(6), 400, 300);
    }

    public void introTwoObjects()
    {
        addObject(new BackTile(105), 692, 255);
        addObject(new BackTile(108), 676, 248);
        addObject(new BackTile(105), 636, 249);
        addObject(new BackTile(108), 624, 247);
        addObject(new BackTile(105), 585, 257);
        addObject(new BackTile(105), 545, 264);
        addObject(new BackTile(105), 509, 255);
        addObject(new BackTile(108), 490, 253);
        addObject(new BackTile(105), 449, 248);
        addObject(new BackTile(102), 410, 255);
        addObject(new BackTile(108), 404, 253);
        addObject(new BackTile(108), 388, 248);
        addObject(new BackTile(105), 348, 249);
        addObject(new BackTile(108), 336, 247);
        addObject(new BackTile(105), 297, 257);
        addObject(new BackTile(105), 257, 264);
        addObject(new BackTile(105), 221, 255);
        addObject(new BackTile(108), 202, 253);
        addObject(new BackTile(105), 162, 248);
        addObject(new BackTile(102), 123, 255);
    }

    public void introThree()
    {
        introPanel = 3;
        setPaintOrder(Credits.class, Cover.class, Statics.class, Front.class, Player.class, Enemies.class, Ground.class, 
            Back.class, Warthog.class, Lives.class, Tree.class, Pelican.class, Background.class, FarBackground.class);
        clearObjects();
        setBackground("ringworld.jpg");
        addObject(new Cover1(1), 400, 300);
        addObject(new Warthog(), 425, 510);
        addObject(new Lives(1), 415, 505);
        addObject(new Tree(), 800, 525);
        addObject(new Credits(9), 400, 300);
        addObject(new Credits(12), 400, 300);
    }

    public void stageOne()
    {
        setPaintOrder(Credits.class, Cover.class, Statics.class, Front.class, Player.class, Enemies.class, Ground.class, 
            Clouds1.class, Pelican.class, Back.class, Background.class, FarBackground.class);
        introPanel = 0;
        Greenfoot.delay(100);
        TitleSong.stop();
        Greenfoot.delay(50);
        StageOneSong.playLoop();
        clearObjects();
        setBackground("ringworld.jpg");
        readTileMapFile();
        readBackgroundMapFile();
        addBackground();
        addTrees();
        addObjects();
        addEnemies();
        addStatics();
        worldObjects = new Objects();
        addObject(worldObjects, 0, -10);
        thePlayer = new Player();
        addObject(thePlayer, 340, 325);
    }

    public void act()
    {
        checkControl();
        checkIntro();
    }

    public void checkIntro()
    {
        introTimer++;
        if (introPanel == 1)
        {
            animateIntroOne();
        }
    }

    public void addTrees()
    {
        addObject(new Tree(), 404, 460);
        addObject(new Tree(), 1244, 100);
        addObject(new Tree(), 2285, 340);
        addObject(new Tree(), 3244, 340);
        addObject(new Tree(), 3644, 60);
    }

    private void addObjects()
    {
        addObject(new Bridge(), 4280, 420);
        addObject(new Warthog(), 170, 85);
        addObject(new SunBeam(), 6300, 10);
        addObject(new SunBeam(50, 100), 4310, 350);
        addObject(new SunBeam(50, 170), 8590, 200);
        addObject(new Health(), 2380, 330);
        addObject(new Checkpoint(), 7080, 400);
    }

    /**
     * Looks for the file "tilemap.txt".  If it cant find it it returns an error message.
     * If it does find it it gets the tile map info and inputs the data into an array 
     * which is sent to addPlatforms().
     */
    private void readTileMapFile()
    {
        InputStream istream =
            getClass().getResourceAsStream("tilemap.txt");
        String[] map = new String[MAP_HEIGHT];

        Scanner in = new Scanner(
                new BufferedReader(
                    new InputStreamReader(istream)));
        for (int i = 0; i < map.length; i++)
        {
            map[i] = in.nextLine();
        }
        in.close();
        addTiles(map);
    }

    /**
     * Looks for the backgroundmap.txt to create the close background.
     * If it cannot find the file it returns an error message to that 
     * effect.
     */
    private void readBackgroundMapFile()
    {
        InputStream istream =
            getClass().getResourceAsStream("backgroundmap.txt");
        String[] backmap = new String[MAP_HEIGHT];

        Scanner in = new Scanner(
                new BufferedReader(
                    new InputStreamReader(istream)));
        for (int i = 0; i < backmap.length; i++)
        {
            backmap[i] = in.nextLine();
        }

        in.close();
        addTiles(backmap);
    }

    /**
     * Creates tiles for building your world.
     */
    private void addTiles(String[] MAP)
    {
        for(int y = 0; y < MAP.length; y++)
        {
            addRow(y, MAP);
        }
    }

    private void addRow(int y, String[] MAP)
    {
        topY = TILE_HEIGHT / 2;
        int tileY = topY + TILE_HEIGHT * y;
        String row = MAP[y];
        for (int x = 0; x < row.length(); x++)
        {
            int tileX = leftX + TILE_WIDTH * x;
            char tileType = row.charAt(x);
            if (tileType == 'B')
            {
                addObject(new Tile(), tileX, tileY);
            }
            if (tileType == 'T')
            {
                addObject(new Tile(98), tileX, tileY);
            }
            if (tileType == 'L')
            {
                addObject(new Tile(97), tileX, tileY);
            }
            if (tileType == 'R')
            {
                addObject(new Tile(99), tileX, tileY);
            }
            if (tileType == 'G')
            {
                addObject(new Tile2(), tileX, tileY);
            }
            if (tileType == 'r')
            {
                addObject(new Tile(101), tileX, tileY);
            }
            if (tileType == 'E')
            {
                addObject(new Tile(108), tileX, tileY);
            }
            if (tileType == 'e')
            {
                addObject(new Tile(107), tileX, tileY);
            }
            if (tileType == 'l')
            {
                addObject(new Tile(100), tileX, tileY);
            }
            if (tileType == '1')
            {
                addObject(new Tile(102), tileX, tileY);
            }
            if (tileType == '2')
            {
                addObject(new Tile(104), tileX, tileY);
            }
            if (tileType == '3')
            {
                addObject(new Tile(103), tileX, tileY);
            }
            if (tileType == '4')
            {
                addObject(new Tile(105), tileX, tileY);
            }
            if (tileType == '5')
            {
                addObject(new Tile(106), tileX, tileY);
            }
            if (tileType == '6')
            {
                addObject(new Tile(109), tileX, tileY);
            }
            if (tileType == '7')
            {
                addObject(new Tile(110), tileX, tileY);
            }
            if (tileType == '-')
            {
                addObject(new BackTile(107), tileX, tileY);
            }
            if (tileType == '=')
            {
                addObject(new BackTile(104), tileX, tileY);
            }
            if (tileType == '0')
            {
                addObject(new BackTile(100), tileX, tileY);
            }
            if (tileType == '/')
            {
                addObject(new BackTile(103), tileX, tileY);
            }
            if (tileType == '+')
            {
                addObject(new BackTile(102), tileX, tileY);
            }
            if (tileType == '_')
            {
                addObject(new BackTile(105), tileX, tileY);
            }
            if (tileType == '|')
            {
                addObject(new BackTile(108), tileX, tileY);
            }
            if (tileType == '?')
            {
                addObject(new BackTile(101), tileX, tileY);
            }
            if (tileType == '<')
            {
                addObject(new BackTile(99), tileX, tileY);
            }
            if (tileType == '>')
            {
                addObject(new BackTile(98), tileX, tileY);
            }
            if (tileType == ',')
            {
                addObject(new BackTile(109), tileX, tileY);
            }
            if (tileType == '.')
            {
                addObject(new BackTile(110), tileX, tileY);
            }
            if (tileType == 'q')
            {
                addObject(new BackTile(113), tileX, tileY);
            }
            if (tileType == 'p')
            {
                addObject(new BackTile(), tileX, tileY);
            }
            if (tileType == 'o')
            {
                addObject(new BackTile(106), tileX, tileY);
            }
        }
    }

    public void addBackground()
    {
        for(int i = 0; i < 5; i++)
        {
            addObject(new Backdrop(), BACKGROUNDWIDTH * i, 378);
            addObject(new Clouds(), BACKGROUNDWIDTH * i, 100);
        }
    }

    /**
     * Creates enemies.
     */
    public void addEnemies()
    {
        addObject(new Enemy(), 1500, 360);
        addObject(new Enemy(), 2500, 240);
        addObject(new Enemy(), 3500, 80);
        addObject(new Enemy(), 4840, 320);
        addObject(new Enemy(), 6640, 320);
        addObject(new Enemy2(), 900, 495);
        addObject(new Enemy2(), 5080, 495);
        addObject(new Enemy2(), 5880, 535);
        addObject(new Enemy2(), 7360, 535);
        addObject(new Enemy2(), 7600, 255);
        addObject(new Enemy2(), 8520, 255);
        addObject(new Sniper(), 6000, 130);
        addObject(new Sniper(), 8200, 170);
        addObject(new Sniper(), 8320, 90);
    }

    public void addStatics()
    {
        addObject(new CheckTile(), 1, 1);
        addObject(new Lives(), 75, 50);
        theLifeCounter = new LifeCounter(totalLives);
        addObject(new LifeCounter(totalLives), 115, 45);
    }

    public Player getPlayer()
    {
        return thePlayer;
    }

    public Objects getObjects()
    {
        return worldObjects;
    }

    public void checkControl()
    {
        if (inControl == true)
        {
            checkInput();
        }
    }

    private void checkInput()
    {
        scrollDirection = 0;
        if (Greenfoot.isKeyDown("left") && !Greenfoot.isKeyDown("right") && thePlayer.canMoveLeft == true)
        {
            scrollDirection = 1;
            shouldScroll = true;
        }
        if (Greenfoot.isKeyDown("right") && !Greenfoot.isKeyDown("left") && thePlayer.canMoveRight == true)
        {
            scrollDirection = -1;
            shouldScroll = true;
        }
        if (Greenfoot.isKeyDown("enter") && titleScreen == true)
        {
            Greenfoot.playSound("chime.mp3");
            addObject(new Cover(1, 1), 400, 300);
            titleScreen = false;
        }
        if (Greenfoot.isKeyDown("enter") && introPanel > 0)
        {
            addObject(new Cover(1, 1), 400, 300);
            introTimer = 0;
            introPanel = 0;
            stageOne();
        }
    }

    public boolean checkScroll()
    {
        return shouldScroll;
    }

    public void resetMap()
    {
        removeObjects(getObjects(null));
        addObject(new Cover(255, -1), 400, 300);
    }

    public void checkLifeTotal()
    {
        if (totalLives < 0)
        {
            inControl = true;
            checkpointReached = false;
            totalLives = 3;
            title();
        }
        else
        {
            stageOne();
        }
    }

    public void changeLifeTotal(int x)
    {
        totalLives = totalLives - x;
        theLifeCounter.changeTotal();

    }

    public void clearObjects()
    {
        removeObjects(getObjects(null));
    }

    public void playTitleSong()
    {
        TitleSong.playLoop();
    }
    
    public void setCheckpoint()
    {
        checkpointReached = true;
    }
}